"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.middleware = exports.validate = exports.validateToken = exports.validateFromEndpoint = void 0;
const axios_1 = __importDefault(require("axios"));
const express_unless_1 = __importDefault(require("express-unless"));
const jsonwebtoken_1 = require("jsonwebtoken");
const pub_key_1 = require("./pub_key");
const supported_version = '050';
const validateFromEndpoint = async (scope, token, app_id) => {
    try {
        const url = scope
            ? `https://ada.cloud.piebits.org/${supported_version}/userops/fetch/self?scope=${scope}`
            : `https://ada.cloud.piebits.org/${supported_version}/userops/fetch/self`;
        const { data } = await axios_1.default.get(url, {
            headers: {
                Authorization: token,
                'x-ada-app-id': app_id,
            },
        });
        return data.user;
    }
    catch (_a) {
        throw new Error('Auth Token Invalid');
    }
};
exports.validateFromEndpoint = validateFromEndpoint;
const validateToken = (token) => {
    try {
        if (token && token.startsWith('Bearer ')) {
            const sanitized_token = token.split('Bearer ')[1];
            const decoded = (0, jsonwebtoken_1.verify)(sanitized_token, pub_key_1.public_key, {
                issuer: 'https://ada.cloud.piebits.org',
                algorithms: ['RS256'],
            });
            return decoded;
        }
        throw new Error('Auth Token should start with bearer');
    }
    catch (_a) {
        throw new Error('Auth Token Invalid');
    }
};
exports.validateToken = validateToken;
const validate = async (props) => {
    if (props.token) {
        if (props.fetchuser) {
            const data = await (0, exports.validateFromEndpoint)(props.scope, props.token, props.app_id);
            return data;
        }
        const data = (0, exports.validateToken)(props.token);
        return data;
    }
    throw new Error('Auth Token is required');
};
exports.validate = validate;
const middleware = (props) => {
    const func = async (req, res, next) => {
        try {
            const token = req.headers.authorization;
            if (!token) {
                res
                    .status(401)
                    .json({
                    type: 'Auth Error',
                    message: 'Auth Token Missing',
                });
                return;
            }
            const data = await (0, exports.validate)({
                ...props,
                token,
            });
            req.ada_user = data;
            next();
        }
        catch (e) {
            res
                .status(401)
                .json({
                type: 'Auth Error',
                message: e.message,
            });
        }
    };
    func.unless = express_unless_1.default;
    return func;
};
exports.middleware = middleware;
